package org.quartz.utils;

/**
 *
 * Skeleton of Quartz classes in order to deserialize it
 *
 * @author Baptiste Mesta
 *
 */
public class DirtyFlagMap implements Map<String, Object>, Cloneable, java.io.Serializable {

    /*
     * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     * Data members.
     * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     */
    private static final long serialVersionUID = 1433884852607126222L;

    private final boolean dirty = false;

    private final Map<String, Object> map;

    public DirtyFlagMap() {
        map = new HashMap<String, Object>();
    }

    public DirtyFlagMap(final int initialCapacity) {
        map = new HashMap<String, Object>(initialCapacity);
    }

    public DirtyFlagMap(final int initialCapacity, final float loadFactor) {
        map = new HashMap<String, Object>(initialCapacity, loadFactor);
    }

    @Override
    public int size() {
        return map.size();
    }

    @Override
    public boolean isEmpty() {
        return map.isEmpty();
    }

    @Override
    public boolean containsKey(final Object key) {
        return map.containsKey(key);
    }

    @Override
    public boolean containsValue(final Object value) {
        return map.containsValue(value);
    }

    @Override
    public Object get(final Object key) {
        return map.get(key);
    }

    @Override
    public Object put(final String key, final Object value) {
        return map.put(key, value);
    }

    @Override
    public Object remove(final Object key) {
        return map.remove(key);
    }

    @Override
    public void putAll(final Map<? extends String, ? extends Object> m) {
        map.putAll(m);
    }

    @Override
    public void clear() {
        map.clear();
    }

    @Override
    public Set<String> keySet() {
        return map.keySet();
    }

    @Override
    public Collection<Object> values() {
        return map.values();
    }

    @Override
    public Set<java.util.Map.Entry<String, Object>> entrySet() {
        return map.entrySet();
    }

    @Override
    public String toString() {
        return "JobDataMap[map=" + (map == null ? "null" : String.valueOf(map)) + "]";
    }
}
